/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "wizard.h"
#include "imagefilter.h"
#include "info.h"



int difficulty=100;


crostiWizard::crostiWizard(QWidget *parent):QWizard(parent),
  colorCount(),
  colorTable(),
  _image(),
  _imageOriginal()
{
     addPage(new introPage);
     addPage(new openImagePage);
     addPage(new convertImagePage(this));
     addPage(new conclusionPage);

     setPixmap(QWizard::LogoPixmap, QPixmap(Info::dataPath()+"/system/image/crosti32.png"));

     setWindowTitle(trUtf8("crosti - Wizard"));
     setGeometry(30,30,400,500);
}


void crostiWizard::accept()
{
  saveRegistry();
  QDialog::accept();
}

void crostiWizard::reject()
{
  clear();
  saveRegistry();
  QDialog::reject();
}






introPage::introPage(QWidget *parent):QWizardPage(parent),
  label(new QLabel(trUtf8("This Wizard helps you to build your personal cross stitch."))),
  chbox(new QCheckBox(trUtf8("Show Wizard at crosti startup")))
{
    setTitle(trUtf8("Intro"));

    label->setWordWrap(true);

    chbox->setChecked(isStartup());
    registerField("wizard",chbox);

    QSpacerItem *spacer=new QSpacerItem(40,20,QSizePolicy::Minimum,QSizePolicy::Expanding);

    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(label);
    layout->addItem(spacer);
    layout->addWidget(chbox);
    setLayout(layout);
}

void introPage::initializePage()
{
  chbox->setChecked(isStartup());
}

conclusionPage::conclusionPage(QWidget *parent):QWizardPage(parent),
  label(new QLabel(trUtf8("Everything is ready to create cross stitch scheme, press \"Finish\" button to create scheme.")))
{
    setTitle(trUtf8("Conclusion"));

    label->setWordWrap(true);

    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(label);
    setLayout(layout);
}







openImagePage::openImagePage(QWidget *parent):QWizardPage(parent),
  imageViewer(new viewer(NULL,this)),
  openImageAct(new QPushButton(trUtf8("Open..."),this))
 {
     setTitle(trUtf8("Picture selection"));
     setSubTitle(trUtf8("Press \"Open...\" button to select picture."));

     openImageAct->setIcon(QIcon(Info::dataPath()+"/system/image/imageopen.png"));
     connect(openImageAct,SIGNAL(clicked()),SLOT(openDialog()));

     QVBoxLayout *layout = new QVBoxLayout;
     layout->addWidget(imageViewer);
     layout->addWidget(openImageAct);
     setLayout(layout);

     registerField("openImage*",imageViewer,"image",SIGNAL(imageChanged(QImage)));
 }

void openImagePage::initializePage()
{
  imageViewer->setImage(QImage());
}

void openImagePage::openDialog()
{
 QSettings rsettings(QApplication::organizationName(),QApplication::applicationName());
 QString picturePath=rsettings.value("picturePath").value<QString>();

 QString fileName=QFileDialog::getOpenFileName(this,trUtf8("Open picture"),picturePath,Info::imageFilter(QImageReader::supportedImageFormats()));
 if(!fileName.isEmpty()){
   QImage image;
   image.load(fileName);
   if(image.isNull()){
     Info::message(trUtf8("Can't open the picture %1.").arg(fileName));
     return;
   }
   imageViewer->setImage(image);

   rsettings.setValue("picturePath",QFileInfo(fileName).canonicalPath());
 }
}





convertImagePage::convertImagePage(crostiWizard *cwp,QWidget *parent):QWizardPage(parent),
  _parent(cwp),
  imageViewer(new viewer(NULL,this)),
  widthSB(new QSpinBox),
  heightSB(new QSpinBox),
  colorSB(new QSpinBox),
  convertImageAct(new QPushButton(trUtf8("Convert..."),this)),
  dmc(),
  _complete(false)
 {

     setTitle(trUtf8("Converting picture"));
     setSubTitle(trUtf8("Press \"Convert...\" button to change picture size and color palette."));

     convertImageAct->setIcon(QIcon(Info::dataPath()+"/system/image/1.png"));
     connect(convertImageAct,SIGNAL(clicked()),SLOT(convertImage()));


     QVBoxLayout *layout = new QVBoxLayout;
     layout->addWidget(imageViewer);


     QGridLayout *gridLayout = new QGridLayout;
     QHBoxLayout *hboxLayout=new QHBoxLayout;
     QSpacerItem *horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);


     QLabel *label1 = new QLabel(trUtf8("Width"));
     widthSB->setMaximum(4000);
     widthSB->setValue(difficulty);
     gridLayout->addWidget(label1,0,0);
     gridLayout->addWidget(widthSB,0,1);

     QLabel *label2 = new QLabel(trUtf8("Height"));
     heightSB->setMaximum(4000);
     gridLayout->addWidget(label2,1,0);
     gridLayout->addWidget(heightSB,1,1);

     QLabel *label3 = new QLabel(trUtf8("Colors"));
     colorSB->setMinimum(2);
     colorSB->setMaximum(255);
     colorSB->setValue(24);
     gridLayout->addWidget(label3,2,0);
     gridLayout->addWidget(colorSB,2,1);

     hboxLayout->addItem(horizontalSpacer);
     hboxLayout->addLayout(gridLayout);
     hboxLayout->addItem(horizontalSpacer);


     layout->addLayout(hboxLayout);


     layout->addWidget(convertImageAct);
     setLayout(layout);
 }


void convertImagePage::initializePage()
{
  setComplete(false);
  QImage image;
  image=field("openImage").value<QImage>();

  widthSB->setValue(difficulty);
  heightSB->setValue(image.height()*widthSB->value()/image.width());
  imageViewer->setImage(image);
}


void convertImagePage::convertImage()
{

 Qt::AspectRatioMode aspectRatio=Qt::KeepAspectRatioByExpanding;
 QImage image=imageViewer->image();
 _parent->setImageOriginal(image);
 QSize sz=QSize(widthSB->value(),heightSB->value());
 float scale=_imageFilter::scaleFit(sz);
 image=image.scaled(sz*scale,aspectRatio,Qt::SmoothTransformation);

 int size=colorSB->value();
 _schemeData schemedata(image);


 if(schemedata.convert(dmc,size)){
   _parent->colorCount.clear();
   _parent->colorTable.clear();

   image=schemedata.image();

   _parent->colorTable.append(schemedata.clT());
   _parent->setImage(image);

   imageViewer->setImage(image);

   setComplete(true);
 }
}





